package pomodoro

import (
	"git.smsvc.net/pomodoro/GoTomato/internal/shared"
	"sync"
	"time"
)

var pomodoroResetChannel = make(chan bool, 1)
var pomodoroPauseChannel = make(chan bool, 1)
var pomodoroResumeChannel = make(chan bool, 1)

var mu sync.Mutex // to synchronize access to shared state

// RunPomodoro iterates the Pomodoro work/break sessions.
func RunPomodoro() {
	mu.Lock()
	shared.Message.Ongoing = true
	shared.Message.Paused = false
	mu.Unlock()

	pomodoroConfig := shared.Message.PomodoroSettings

	for session := 1; session <= pomodoroConfig.Sessions; session++ {
		shared.Message.Session = session
		shared.Message.Mode = "Work"
		if !startTimer(pomodoroConfig.Work) {
			break
		}
		if session == pomodoroConfig.Sessions {
			shared.Message.Mode = "LongBreak"
			if !startTimer(pomodoroConfig.LongBreak) {
				break
			}
		} else {
			shared.Message.Mode = "ShortBreak"
			if !startTimer(pomodoroConfig.ShortBreak) {
				break
			}
		}
		shared.Message.Mode = "End"
	}

	time.Sleep(time.Second)

	mu.Lock()
	shared.Message.Ongoing = false
	shared.Message.Paused = false
	mu.Unlock()

	shared.Message.Mode = "Idle"
	shared.Message.Session = 0
	shared.Message.TimeLeft = shared.Message.PomodoroSettings.Work
}

// Stops and resets the running Pomodoro
func ResetPomodoro() {
	shared.Message.Mode = "Idle"
	shared.Message.Session = 0
	shared.Message.TimeLeft = shared.Message.PomodoroSettings.Work
	pomodoroResetChannel <- true
}

func PausePomodoro() {
	mu.Lock()
	shared.Message.Paused = true
	mu.Unlock()

	pomodoroPauseChannel <- true
}

func ResumePomodoro() {
	mu.Lock()
	shared.Message.Paused = false
	mu.Unlock()
	pomodoroResumeChannel <- true
}

func IsPomodoroOngoing() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.Message.Ongoing
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.Message.Paused
}
